# pylint: skip-file

import sys
import os
import unittest

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))))
sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))

from bobot import bobot
from cases.text import simpleText, regexText, arrayTextOr, arrayTextAnd

from cases.response.string import *
from cases.response.text import *
from cases.response.photo import *
from cases.after import afterTests, testResultOkCalls
from cases.commands import *

from constants import DEV_BOT_TOKEN

def getBot(rules):
    bot = bobot.init(DEV_BOT_TOKEN, 'TestBot')
    bot.setWebhook(None)
    bot.rule(rules)
    return bot

class RuleTestCases(unittest.TestCase):
    def testSimpleText(self):
        bot = getBot(simpleText.rules)
        self.assertTrue(simpleText.check(bot))

    def testRegexp(self):
        bot = getBot(regexText.rules)
        self.assertTrue(regexText.check(bot))

    def testArrayOr(self):
        bot = getBot(arrayTextOr.rules)
        self.assertTrue(arrayTextOr.check(bot))

    def testArrayAnd(self):
        bot = getBot(arrayTextAnd.rules)
        self.assertTrue(arrayTextAnd.check(bot))

class RuleAfterTestCases(unittest.TestCase):
    def testAfterCalls(self):
        bot = getBot(afterTests.rules)
        afterTests.checkAfter(bot)
        self.assertNotEqual(afterTests.before, afterTests.after)
        self.assertEqual(afterTests.awaits, afterTests.after)

    def testResultOkCalls(self):
        bot = getBot(testResultOkCalls.rules)
        testResultOkCalls.checkAfter(bot)
        self.assertNotEqual(testResultOkCalls.before, testResultOkCalls.after)
        self.assertEqual(testResultOkCalls.awaits, testResultOkCalls.after)

class RuleCommandTestCases(unittest.TestCase):
    def testCommandWithSlash(self):
        bot = getBot(commandWithSlash.rules)
        self.assertTrue(commandWithSlash.check(bot))
    
    def testCommandWithSlashAndUserName(self):
        bot = getBot(commandWithSlashAndUserName.rules)
        self.assertTrue(commandWithSlashAndUserName.check(bot))
    
    def testCommandStandaloneWithSlash(self):
        bot = getBot(commandStandaloneWithSlash.rules)
        self.assertTrue(commandStandaloneWithSlash.check(bot))
    
    def testCommandStandaloneWithSlashAndUserName(self):
        bot = getBot(commandStandaloneWithSlashAndUserName.rules)
        self.assertTrue(commandStandaloneWithSlashAndUserName.check(bot))

class TextResponseTestCases(unittest.TestCase):
    def testResponseAsText(self):
        bot = getBot(responseAsText.rules)
        self.assertTrue(responseAsText.check(bot))

    def testResponseInterpolateText(self):
        bot = getBot(responseInterpolateText.rules)
        self.assertTrue(responseInterpolateText.check(bot))

    def testResponseInterpolateUserName(self):
        bot = getBot(responseInterpolateUserName.rules)
        self.assertTrue(responseInterpolateUserName.check(bot))

    def testResponseInterpolateDate(self):
        bot = getBot(responseInterpolateDate.rules)
        self.assertTrue(responseInterpolateDate.check(bot))

    def testResponseAsTextDict(self):
        bot = getBot(responseAsTextDict.rules)
        self.assertTrue(responseAsTextDict.check(bot))

    def testResponseAsTextDictOptions(self):
        bot = getBot(responseAsTextDictOptions.rules)
        self.assertTrue(responseAsTextDictOptions.check(bot))

    def testResponseAsTextObject(self):
        bot = getBot(responseAsTextObject.rules)
        self.assertTrue(responseAsTextObject.check(bot))

    def testResponseAsTextObjectOptions(self):
        bot = getBot(responseAsTextObjectOptions.rules)
        self.assertTrue(responseAsTextObjectOptions.check(bot))

    def testResponseInterpolateBody(self):
        bot = getBot(responseInterpolateBody.rules)
        self.assertTrue(responseInterpolateBody.check(bot))
  
class PhotoResponseTestCases(unittest.TestCase):
    def testResponsePhotoWithoutCaption(self):
        bot = getBot(responsePhotoWithoutCaption.rules)
        self.assertTrue(responsePhotoWithoutCaption.check(bot))

    def testResponsePhotoWithCaption(self):
        bot = getBot(responsePhotoWithCaption.rules)
        self.assertTrue(responsePhotoWithCaption.check(bot))

    def testResponsePhotoErrorFileNotFound(self):
        bot = getBot(responsePhotoErrorFileNotFound.rules)
        self.assertTrue(responsePhotoErrorFileNotFound.handlerError(bot, FileNotFoundError))

    def testResponsePhotoWithoutCaptionAsPhoto(self):
        bot = getBot(responsePhotoWithoutCaptionAsPhoto.rules)
        self.assertTrue(responsePhotoWithoutCaptionAsPhoto.check(bot))

    def testResponsePhotoWithCaptionAsPhoto(self):
        bot = getBot(responsePhotoWithCaptionAsPhoto.rules)
        self.assertTrue(responsePhotoWithCaptionAsPhoto.check(bot))

    def testResponsePhotoErrorFileNotFoundAsPhoto(self):
        bot = getBot(responsePhotoErrorFileNotFoundAsPhoto.rules)
        self.assertTrue(responsePhotoErrorFileNotFoundAsPhoto.handlerError(bot, FileNotFoundError))

class BotTestCases(unittest.TestCase):
    def testToken(self):
        bot = bobot.init('a')
        self.assertEqual('a', bot.getToken())

    def testDifferentInstances(self):
        bot1 = bobot.init('a')
        bot2 = bobot.init('b')
        self.assertTrue(bot1.getToken() != bot2.getToken())

unittest.main()



"""
    User responses module.
    Contains:
        Message
        Text
        Keyboard
        Location
        File
        Photo
        Contact
"""

from bobot.utils import instanceof, omit, pickCompat
from bobot.Errors import ResponseFormatError, ResponseMessageError

class Message:
    "Main Message class container"

    method = 'sendMessage'

    def __init__(self, **options):
        self.options = options

    def getData(self, data):
        # pylint: disable=unused-argument
        "Return data to send with method"

        return []

    def getOptions(self):
        "Return options dict"

        options = self.options or {}
        options.update({
            'disable_notification': self.options.get('sielent', False),
            'reply_to_message_id': self.options.get('replyId', None),
        })

        if self.options.get('markup'):
            options.update({'reply_markup': self.options.get('markup', None)})

        return options

    def run(self, bot, update):
        "Apply update to response body"

        message = update.get('message', {})
        text = message.get('text')
        sender = message.get('from', {})
        senderId = sender.get('id')
        chatId = message.get('chat', {}).get('id')

        responseId = chatId if not self.options.get('direct') else senderId

        data = {
            'text': text,
            'username': sender.get('username'),
            'first_name': sender.get('first_name'),
            'second_name': sender.get('second_name')
        }

        methodArgs = self.getData(data)

        if not instanceof(methodArgs, [dict, tuple]):
            methodArgs = [methodArgs]

        sendArguments = [responseId] + list(methodArgs)

        options = self.getOptions()
        options = {'options': options}

        return getattr(bot, self.method)(*sendArguments, **options)

class Text(Message):
    "Text response container"
    # pylint: disable=missing-docstring

    method = 'sendMessage'

    def __init__(self, text, **options):
        super().__init__(**options)
        self.text = text

    def getData(self, data):
        text = self.text

        if self.options.get('interpolate'):
            text = self.text.format(
                text=data.get('text'),
                first_name=data.get('first_name'),
                username=data.get('username'),
                second_name=data.get('second_name'))

        return text

    def getOptions(self):
        options = super().getOptions()
        print('options: {}'.format(options))

        additional = {
            'disable_web_page_preview': self.options.get('disableWebPreview', False),
        }

        options.update(additional)

        if self.options.get('format'):
            options.update({'parse_mode': self.options.get('format')})

        if self.options.get('markup'):
            options.update({'reply_markup': self.options.get('markup')})

        return options

class Keyboard(Text):
    "Keyboard response container"
    method = 'sendKeyboard'
    # pylint: disable=missing-docstring

    @staticmethod
    def __toKeyboard(row):
        def toButton(btn):
            if isinstance(btn, dict) and btn['text']:
                return btn
            else:
                return {
                    'text': str(btn)
                }
        return list(map(toButton, row))

    def __init__(self, text, keyboard=None, **options):
        #pylint: disable=redefined-variable-type
        super().__init__(text, **options)
        self.text = text if keyboard else text.get('text')
        self.keyboard = keyboard if keyboard else text.get('keyboard', [])

        self.keyboard = list(map(self.__toKeyboard, self.keyboard))

        if isinstance(self.keyboard, list):
            self.keyboard = {
                'keyboard': self.keyboard
            }

        if options.get('autohide') != None:
            self.keyboard['one_time_keyboard'] = options.get('autohide')

        if options.get('resize') != None:
            self.keyboard['resize_keyboard'] = options.get('resize')

    def getData(self, data):
        return super().getData(data), self.keyboard

class Location(Message):
    "Location response container"
    # pylint: disable=missing-docstring
    method = 'sendLocation'

    def __init__(self, lat, lon, **options):
        super().__init__(**options)
        self.lat = lat
        self.lon = lon

    def getData(self, data):
        return self.lat, self.lon

class Sticker(Message):
    "Sticker response container"
    # pylint: disable=missing-docstring
    method = 'sendSticker'

    def __init__(self, sticker, **options):
        super().__init__(**options)
        self.sticker = sticker

    def getData(self, data):
        return self.sticker


class File(Message):
    "File response container"
    # pylint: disable=missing-docstring

    method = 'sendDocument'

    def __init__(self, file, caption=None, **options):
        super().__init__(**options)
        self.file = file
        self.caption = caption

    def getData(self, data):
        return self.file, self.caption


class Photo(File):
    "Photo response container"
    method = 'sendPhoto'

class Document(File):
    "Document response container"
    method = 'sendDocument'

class Voice(File):
    "Voice response container"
    method = 'sendVoice'

class Audio(File):
    "Audio response container"
    method = 'sendAudio'

class Video(File):
    "Video response container"
    method = 'sendVideo'

class Contact(Message):
    "Contact response container"
    # pylint: disable=missing-docstring
    method = 'sendContact'

    def __init__(self, phone, first, last='', **options):
        super().__init__(**options)
        self.phone = phone
        self.first = first
        self.last = last

    def getData(self, data):
        return self.phone, self.first, self.last

def getAction(actionType, data):
    "Calculates action by actionType in response description"
    # pylint: disable=missing-docstring,unnecessary-lambda,redefined-variable-type,too-many-locals,too-many-branches
    response = None

    if actionType == 'text':
        if isinstance(data, dict):
            text = data.get('text', '')
            data = omit(data, ['text'])
            response = Text(text, **data)
        else:
            text = data
            response = Text(text)
    elif actionType == 'location':
        if isinstance(data, dict):
            lat = data.get('lat')
            lon = data.get('lon')
            options = omit(data, ['lat', 'lon'])
            response = Location(lat, lon, **options)
        elif isinstance(data, list):
            response = Location(data[0], data[1])
        else:
            raise ResponseFormatError('Invalid format at Location response')
    elif actionType == 'keyboard':
        if isinstance(data, dict):
            optionNames = ['autohide', 'resize']
            options = pickCompat(data, optionNames)
            keyboard = omit(data, optionNames)
            response = Keyboard(keyboard, **options)
        else:
            raise ResponseFormatError('Invalid format at Keyboard response')
    elif actionType == 'photo':
        if isinstance(data, dict):
            photo = data.get('photo')
            caption = data.get('caption')
            options = omit(data, ['photo', 'caption'])
            response = Photo(photo, caption, **options)
        else:
            response = Photo(data)
    elif actionType == 'sticker':
        if isinstance(data, dict):
            sticker = data.get('sticker')
            options = omit(data, ['sticker'])
            response = Sticker(sticker, **options)
        else:
            response = Sticker(data)
    elif actionType == 'contact':
        if isinstance(data, dict):
            phone = data.get('phone')
            name = data.get('firstName')
            lastName = data.get('lastName')
            options = omit(data, ['phone', 'firstName', 'lastName'])
            response = Contact(phone, name, lastName, **options)
        else:
            raise ResponseFormatError('Invalid format at Contact response')
    elif actionType == 'voice' or actionType == 'audio':
        if isinstance(data, dict):
            file = data.get(actionType)
            caption = data.get('caption')
            options = omit(data, ['voice', 'caption', 'audio'])
            response = Voice(file, caption, **options) if actionType == 'voice' else Audio(file, caption, **options)
        else:
            typeOfResponse = 'Voice' if actionType == 'voice' else 'Audio'
            raise ResponseFormatError('Invalid format at {} response'.format(typeOfResponse))
    elif actionType == 'video':
        if isinstance(data, dict):
            file = data.get('video')
            caption = data.get('caption')
            options = omit(data, ['caption', 'video'])
            response = Video(file, caption, **options)
        else:
            raise ResponseFormatError('Invalid format at Video response')
    else:
        raise ResponseMessageError('Invalid response type: "{}"'.format(actionType))

    def action(response):
        return lambda bot, update: response.run(bot, update)

    return action(response)

class Response():
    "Response class"

    __map = {
        'text': Text,
        'keyboard': Keyboard,
        'location': Location,
        'photo': Photo,
        'sticker': Sticker,
        'audio': Audio,
        'voice': Voice
    }

    def __init__(self, responses):
        self.body = responses
        self.actions = []

        if not isinstance(responses, list):
            responses = [responses]

        for response in responses:
            self.addAction(response)

    def addAction(self, response):
        "Adds action to actions list"
        if len(response) > 1:
            raise ResponseFormatError('Response can hold only one action')

        responseKey = list(response.keys()).pop()
        respBody = response[responseKey]

        if responseKey in self.__map:
            self.action = getAction(responseKey, respBody)
        else:
            raise ResponseFormatError('Invalid response action: "{}"'.format(responseKey))

    def run(self, bot, update):
        "Runs response actions"
        return self.action(bot, update)



# pylint: disable=wrong-import-position
import sys
import os

PACKAGE_PARENT = '..'
SCRIPT_DIR = os.path.dirname(os.path.realpath(os.path.join(os.getcwd(), os.path.expanduser(__file__))))
sys.path.append(os.path.normpath(os.path.join(SCRIPT_DIR, PACKAGE_PARENT)))

from flask import Flask, request
from bobot import bobot
from constants import DEV_BOT_TOKEN
import testcases

mode = sys.argv[-1]

app = Flask('bot-test')

bot = bobot.init(DEV_BOT_TOKEN, 'TestBot')
bot.setWebhook(None)
bot = testcases.assign(bot)

@app.route('/')
def hello():
    return 'It is alive!'

@app.route('/about')
def about():
    res = str(bot.about())
    return res

@app.route('/updates')
def updates():
    res = str(bot.getUpdates())
    return res

@app.route('/bot', methods=['GET', 'POST'])
def test():
    body = request.get_json()
    bot.process(body)
    return ''

if __name__ == "__main__":
    app.run(port=8002, debug={mode is 'debug'})



#coding: utf8

"""
	Bobot: Simple Telegram Bot API Wrapper for Python3.5
	https://github.com/zefirka/bobot
"""

# А мир был чудесный, как сопля на стене
# А город был хороший, словно крест на спине
# А день был счастливый-как слепая кишка
# А он увидел Солнце...

from .bobot import init
from .Rule import Rule
from .Parser import Parser
from .Response import Response, Message, Text, Keyboard, Location, File, Photo, Contact, Audio, Voice, Sticker



"Requests module"

import requests

def call(url, method='GET', data={}, headers={}, files=None):
    """
        Calls URL:
        @param {str}  url
        @param {str}  [method]
        @param {dict} [data]
        @param {dict} [headers]
        @returns {str}
    """
    if method is 'GET':
        request = requests.get(url, params=data, headers=headers)
    else:
        request = requests.post(url, data=data, headers=headers, files=files)

    request.encoding = 'utf-8'

    return request.text

def get(url, data={}, headers={}):
    "Calls url with method GET"
    return call(url, 'GET', data, headers)

def post(url, data={}, headers={}, files={}):
    "Calls url with method POST"
    return call(url, 'POST', data, headers, files)



"Errors module"

class RuleNameError(Exception):
    "Error in name of rule"
    pass

class ParsingError(Exception):
    "Error while parsing of message"
    pass

class ResponseFormatError(Exception):
    "Error in response format"
    pass

class ResponseMessageError(Exception):
    "Error in response message description"
    pass

class MessageError(Exception):
    "Exception for error in message content"
    pass

        


"Module contains Parser"

class Parser(object):
    "Parser class"

    def __init__(self, splitter=' '):
        self.splitter = splitter

    def parse(self, text):
        "Parses given text"
        return text.split(self.splitter)



"""
    Main bot module
    Includes class Bot
"""

import re
from json import loads, dumps

from bobot.Rule import Rule
from bobot.Response import Response
from bobot.Errors import MessageError
from bobot.req import get, post
from bobot.utils import getFile

__api = 'https://api.telegram.org/bot{token}/{method}'

def init(token, name=None):
    'Initialize'

    bot = Bot(token, name)
    return bot

def call(token, method, data={}, files=None):
    "Calls telegram API"
    url = __api.format(token=token, method=method)

    if files:
        response = loads(post(url, data, None, files))
    else:
        response = get(url, data)

    return response

def caller(method, **kwargs):
    "Telegram API calling decorator"

    method = kwargs.get('method', method)

    def sendFunciton(token, *args):
        "Calling function"
        return call(token, method, *args)

    def callerDecorator(dataFunction):
        "Caller decorator"

        def callerFunction(self, chatId=None, *args, **options):
            "API Caller"
            argumentsOrder = kwargs.get('arguments')
            requiredArguments = kwargs.get('required')

            if argumentsOrder and requiredArguments:
                for index, argument in enumerate(argumentsOrder):
                    if requiredArguments.get(argument) and not args[index]:
                        raise requiredArguments.get(argument)

            if kwargs.get('static', False) is True:
                callArgs = dataFunction(chatId, *args)
            else:
                callArgs = dataFunction(*args)

            if isinstance(callArgs, dict):
                callArgs = [callArgs]

            if kwargs.get('static', False) is not True:
                callArgs[0]['chat_id'] = chatId

            if options.get('options', False):
                callArgs[0].update(options.get('options'))

            return sendFunciton(self.getToken(), *callArgs)

        return callerFunction

    return callerDecorator

class Bot(object):
    # pylint: disable=no-self-argument
    """
        Bot class
    """
    def __addRule(self, rule):
        if not isinstance(rule, Rule):
            raise Exception('rule is not Rule instance')

        self.rules.append(rule)
        return self.rules

    #####################################################
    #####################################################

    def __init__(self, token, name=None):
        self.__info = None
        self.__token = token
        self.rules = []
        self.clients = {}
        self.name = name

    def about(self):
        "Returns information about bot"

        if self.__info:
            return self.__info

        info = call(self.__token, 'getMe')
        info = loads(info)
        self.__info = info
        return info.get('result')

    @caller('sendMessage',
            arguments=['text'],
            required={
                'text': MessageError('Specify message\'s text')
            })
    def sendMessage(text):
        "Sends text to user"

        data = {
            'text': text
        }

        return data

    @caller('sendSticker',
            arguments=['sticker'],
            required={
                'sticker': MessageError('Specify StickerID')
            })
    def sendSticker(stickerId):
        "Sends stricker to user"

        data = {
            'sticker': stickerId
        }

        return data

    @caller('sendPhoto')
    def sendPhoto(photo, caption=None):
        "Sends photo to user"

        data = {}
        if caption:
            data['caption'] = caption

        if isinstance(photo, str):
            photo = getFile(photo)

        file = {'photo': photo}

        return data, file

    @caller('sendDocument')
    def sendDocument(doc, caption=None):
        "Sends document"

        data = {}
        if caption:
            data['caption'] = caption

        if isinstance(doc, str):
            doc = getFile(doc)

        file = {'document': doc}

        return data, file

    @caller('sendAudio')
    def sendAudio(audio, caption=None):
        "Sends audio"

        data = {}
        if caption:
            data['caption'] = caption

        if isinstance(audio, str):
            audio = getFile(audio)

        file = {'audio': audio}

        return data, file

    @caller('sendVoice')
    def sendVoice(voice, caption=None):
        "Sends voice"

        data = {}
        if caption:
            data['caption'] = caption

        if isinstance(voice, str):
            voice = getFile(voice)

        file = {'voice': voice}

        return data, file

    @caller('sendVideo')
    def sendVideo(video, caption=None):
        "Sends video"

        data = {}
        if caption:
            data['caption'] = caption

        if isinstance(video, str):
            video = getFile(video)

        file = {'video': video}

        return data, file

    @caller('sendLocation')
    def sendLocation(lat, lon):
        "Sends location"

        return {
            'latitude': lat,
            'longitude': lon
        }

    @caller('sendContact')
    def sendContact(phone, name, secondName=None):
        "Send contact to user"
        return {
            'phone_number': phone,
            'first_name': name,
            'secon_name': secondName
        }

    @caller('sendMessage')
    def sendKeyboard(text, keyboard=None):
        "Sends keyboard to user"

        if not keyboard:
            keyboard = text
            text = text.pop('text', '')

        if not text:
            raise MessageError('Specify text message')

        board = dumps(keyboard)

        return {
            'text': text,
            'reply_markup': board
        }

    def process(self, update):
        """
            Process update by bot's rules
            @public
            @param {dict} update
        """
        result = []

        if not len(self.rules):
            return None

        for rule in self.rules:
            result.append(rule.execRule(self, update))

        return list(filter(bool, result))

    def rule(self, rules):
        """
            Assign rules to bot
            @public
            @param {list[Rule]|Rule} rules
        """

        if not isinstance(rules, list):
            rules = [rules]

        for rule in rules:
            self.__addRule(rule)

    def on(self, match, response, flags=0):
        "Subscribes to matching"

        if isinstance(response, dict) or isinstance(response, list):
            response = Response(response)

        self.rule(Rule({
            'name': match,
            'match': lambda text: bool(re.compile(match, flags).match(text)),
            'response': response
        }))

    @caller('getUpdates', static=True)
    def getUpdates(limit=None, offset=None):
        "Call getUpdates method"
        return {
            'limit': limit,
            'offset': offset
        }

    def register(self, user, registerInfo={'id': 'id'}):
        "Registers client to bot memory"

        result = {}
        for key in registerInfo:
            result[key] = user.get(registerInfo[key])
        return result


    @caller('sendChatAction')
    def sendChatAction(action):
        "Send chatAction object"

        return {
            'action': action
        }

    @caller('setWebhook', static=True)
    def setWebhook(url, certificate=None):
        "Setting up Telegram Webhook for given bot"

        data = {
            'url': url
        }

        if certificate:
            data['certificate'] = certificate

        return data

    def getToken(self):
        "Returns token"

        return self.__token



"Utils module"

def isFn(fn):
    """
        Is argument a function

        @public
        @param {*} fn
        @return {bool}
    """
    return hasattr(fn, '__call__')

def execValue(val, args=None):
    """
        Returns value if it's not a function, else returns value called with arguments (None by default)

        @public
        @param {*|function}     val
        @param {*}              [args]
        @returns {*}
    """
    return val(*args) if isFn(val) else val

def some(pred, lst):
    #pylint: disable=missing-docstring
    return bool(len(list(filter(pred, lst))))

def flatten(lst):
    """
        Flattens list recursively

        @public
        @param {list} lst
        @return {list}
    """
    if lst == []:
        return lst
    if isinstance(lst[0], list):
        return flatten(lst[0]) + flatten(lst[1:])
    return lst[:1] + flatten(lst[1:])

def getFile(f):
    """
        Returns opened file

        @public
        @param {str} f
        @return file
    """
    return open(f, 'rb')

def instanceof(obj, instances):
    "Checks does object instance of some of given instances"
    lst = filter(lambda instance: isinstance(obj, instance), instances)
    return bool(len(list(lst)))

def omit(obj, keys):
    """
        Remove keys from dict by names
        @param {dict} obj
        @param {list[str]} keys
        @return {dict}

    """
    obj = obj.copy()
    for key in keys:
        if obj.get(key):
            del obj[key]
    return obj

def pickCompat(obj, keys):
    """
        Forming dict from given one and list of keys
        @param {dict} obj
        @param {list[str]} keys
        @return {dict}
    """
    result = {}
    for key in keys:
        result[key] = obj.get(key)
    return result

def someOf(*preds):
    "Return predicate returns true if SOME OF GIVEN PREDICATS return true"

    def predicate(arg):
        #pylint: disable=missing-docstring
        return some(lambda pred: pred(arg), filter(bool, preds))
    return predicate



"Module contains test cases for test server"

import re

from bobot.Rule import Rule
from bobot.Parser import Parser
from bobot.Response import Voice, Video

from cases.files import rules as filerules

parser = Parser()

def gt(x):
    return lambda y: y > x

def lt(x):
    return lambda y: y < x

actionTest = lambda bot, upd, body: bot.sendMessage(upd.get('message').get('from').get('id'), 'action completed bro')

def kb(bot, upd, body):
    # pylint: disable=unused-argument
    "Send keyboard"

    userId = upd.get('message').get('from').get('id')
    bot.sendKeyboard(userId, 'sosi', {
        'resize_keyboard': True,
        'keyboard': [
            [
                {'text': 'alpha'},
                {'text': 'betta'}
            ]
        ],
        'one_time_keyboard': True
    })

testkb = {
    'resize_keyboard': True,
    'text': 'hallo bro',
    'keyboard': [
        [
            'alpha',
            'betta'
        ]
    ],
    'one_time_keyboard': True
}

rules = [
    Rule({
        'match': ['alpha', 'betta'],
        'response': 'alpha or betta was sent'
    }),
    Rule({
        'match': 'action',
        'action': actionTest
    }),
    Rule({
        'name': 'case',
        'match': 'yo-yo',
        'response': [
            {
                'text': {
                    'text': 'Salam, {username}!',
                    'interpolate': True
                }
            },
            {
                'text': 'kase',
            },
            {
                'keyboard': testkb
            }
        ]
    }),
    Rule({
        'match': lambda text: 'parse' in text,
        'parse': parser.parse,
        'response': '{body}'
    }),
    Rule({
        'match': re.compile(r'^regexp'),
        'response': 'regexp included'
    }),
    Rule({
        'parse': lambda text: text.isdigit() and int(text),
        'match': Rule.all(lt(100), gt(50)),
        'response': '{text} less than 100 and greater than 50'
    }),
    Rule({
        'match': Rule.all(re.compile(r'^666'), re.compile(r'.*hell$')),
        'response': 'DAMN HELL!!!'
    }),
    Rule({
        'match': 'keyboard',
        'action': kb
    }),
    Rule({
        'match': re.compile(r'where\s*am\s*i\??'),
        'response': [{
            'location': [40.781984, 43.886827],
        }, {
            'text': 'Home, sweet home'
        }]
    })
]

def wordsCount(message):
    return len(message.split(' '))


def assign(bot):
    bot.rule(Rule({
        'match': lambda length: length > 2,
        'parse': wordsCount,
        'response': 'Words count was: {body}'
    }))

    bot.on('video', Video('./test/files/video.mp4', 'sosi'))

    bot.on('test', 'responses from on method as string')
    bot.on(r'^test?$', 'responses from on method as regexp')
    bot.on('html', {
        'text': {
            'text': '<pre>И так я \nтоже могу\n:3</pre>',
            'format': 'html'
        }
    })
    bot.on('jazz', {
        'sticker': 'BQADAgADBQADIyIEBsnMqhlT3UvLAg'
    })
    bot.rule(rules)
    bot.rule(filerules)

    bot.rule(Rule({
        'command': 'command',
        'response': 'Commands do work'
    }))

    voice = Voice('./test/files/voice.opus', 'caption', duration=3)

    bot.on('voice', voice)

    return bot



from distutils.core import setup

setup(
  name = 'bobot',
  packages = ['bobot'],
  version = '0.1.4',
  description = 'Telegram Bot API Wrapper',
  author = 'Trdat Mkrtchyan',
  author_email = 'wombtormb@gmail.com',
  url = 'https://github.com/zefirka/bobot',
  download_url = 'https://github.com/zefirka/bobot/archive/v0.1.4.tar.gz',
  keywords = ['telegram', 'bot', 'api'],
  classifiers = []
)








"Module containes Rule class"

import re

from bobot.Errors import RuleNameError
from bobot.utils import execValue, isFn, instanceof, someOf
from bobot.Response import Response, Message

def isResponseClass(i):
    "Checks is item is response class"
    return instanceof(i, [Response, Message])

__retype = type(re.compile('re'))

def getMatcher(match):
    "Return matcher by `match` value of Rule"

    def fn(text):
        "Mathing by function"
        return match(text)

    def lst(text):
        "Mathing by list of mathers"
        for submatch in match:
            submatcher = getMatcher(submatch)

            if submatcher(text):
                return True
        return False

    def eq(text):
        "Mathing by equality"
        return text == match

    def rematch(text):
        "Mathing as regex"
        return bool(match.match(text))

    if isFn(match):
        return fn

    if isinstance(match, list):
        return lst

    if isinstance(match, str) or isinstance(match, int) or isinstance(match, float):
        return eq

    if isinstance(match, __retype):
        return rematch


def formatResponse(response, update, body):
    "Formats given response with update's data"

    message = update.get('message', {})
    text = message.get('text')
    sender = message.get('from', {})
    senderName = sender.get('first_name')
    senderSecondName = sender.get('second_name')
    username = sender.get('username')
    date = message.get('date')

    return response.format(
        text=text,
        username=username,
        body=body,
        name=senderName,
        secondName=senderSecondName,
        date=date
    )

def getResponder(response, bot, update, body):
    "Calculates responder for given response by type"

    if isinstance(response, dict):
        response = Response(response)

    if isResponseClass(response):
        return response.run(bot, update)

    if isinstance(response, list):
        result = []
        for res in response:
            result.append(getResponder(res, bot, update, body))
        return result

    response = execValue(response, [body, bot])
    response = formatResponse(response, update, body)

    userId = update.get('message', {}).get('from', {}).get('id')
    return bot.sendMessage(userId, response)

class Rule(object):
    "Rules class"

    __alowedRules = ['name', 'match', 'response', 'command', 'action', 'parse', 'transform', 'after', 'command']

    @staticmethod
    def all(*args):
        "Helper method"

        def matcher(text):
            "Matcher"
            for match in args:
                matcher = getMatcher(match)
                if not matcher(text):
                    return False
            return True
        return matcher

    def __init__(self, d):
        self.match = None
        self.command = None

        for key in d:
            if key in self.__alowedRules:
                setattr(self, key, d[key])
            else:
                raise RuleNameError('Rule property: "{}" is invalid'.format(key))

        if not self.match and not self.command:
            raise Exception('Rule have not matcher and command at same time')

    def getCommandMatcher(self, bot):
        """
            Returns matcher for any type of command e.g.
            /commandName
            /commandName@bot_name
        """

        if self.command:
            return getMatcher(re.compile(r'^/{command}(@{name})?$'.format(command=self.command, name=bot.name), re.I))

    def execRule(self, bot, update):
        """
            Executes rule

            @public
            @param {Bot} bot
            @param {dict} update
        """
        message = update.get('message', {})
        text = message.get('text')
        sender = message.get('from', {})
        senderId = sender.get('id')

        body = text

        if hasattr(self, 'parse'):
            #pylint: disable=broad-except
            try:
                body = self.parse(body)
            except Exception as error:
                print('Parsing Error:')
                print(error)

        matcher = lambda x: False

        if self.match:
            matcher = getMatcher(self.match)

        if self.command:
            matcher = someOf(matcher, self.getCommandMatcher(bot))

        if matcher(body):
            if hasattr(self, 'transform'):
                body = self.transform(body)

            if hasattr(self, 'register') and not bot.clients.get(senderId):
                bot.clients[senderId] = bot.register(sender, self.register)

            if hasattr(self, 'action'):
                self.action(bot, update, body)

            if hasattr(self, 'response'):
                responseResult = getResponder(self.response, bot, update, body)

                if hasattr(self, 'after'):
                    self.after(responseResult, bot, update)

                return responseResult

    def on(self, text, action):
        "Create rule assigned on text"
        if not isinstance(self.match, dict):
            self.match = {
                'value': self.match
            }
            self.match.setdefault('callbacks', {})
            self.match['callbacks'][text] = action

    def addMatching(self, matching):
        "Add new matching"

        if isinstance(self.match, list):
            self.match.append(matching)
        else:
            self.match = [self.match, matching]
